// OpenCV
#include <opencv2/opencv.hpp>
#include <opencv2/core.hpp>
#include <opencv2/highgui.hpp>

// ARマーカー関連
#include <opencv2/aruco.hpp>

// ROS関連
#include <ros/ros.h>
#include <ros/node_handle.h>
#include <cv_bridge/cv_bridge.h>
#include <image_transport/image_transport.h>
#include <sensor_msgs/Image.h>
#include <sensor_msgs/CompressedImage.h>

class arMarkerRecognizer{
public:
	ros::NodeHandle nh_;
	image_transport::ImageTransport it_;
	image_transport::Publisher image_pub_;
	static const int cnt = 0;

	arMarkerRecognizer()
		: it_(nh_)
	{
		image_pub_ = it_.advertise("/camera/image_raw", 10);
	}

	void recognizeArMarker(cv::Mat frame)
	{
		cv::Ptr<cv::aruco::DetectorParameters> parameters = cv::aruco::DetectorParameters::create();
		cv::Ptr<cv::aruco::Dictionary> dictionary = cv::aruco::getPredefinedDictionary(cv::aruco::DICT_4X4_100);

		for(int i = 0; i < cnt + 1; i++){
			std::vector<std::vector<cv::Point2f>> corners, rejectedCandidates;
			std::vector<int> ids;

			// ArUcoマーカの検出
			cv::aruco::detectMarkers(frame, dictionary, corners, ids, parameters, rejectedCandidates);
	
			if(ids.size() > 0){
				// ArUcoマーカの検出結果の描画
				cv:: aruco::drawDetectedMarkers(frame, corners, ids);
			}
		}
	}

	void imagePublisher(cv::Mat frame)
	{
		sensor_msgs::ImagePtr msg = cv_bridge::CvImage(std_msgs::Header(), sensor_msgs::image_encodings::BGR8, frame).toImageMsg();
		image_pub_.publish(msg);
	}
};

int main(int argc, char* argv[]){
	ros::init(argc, argv, "image_publisher");

	cv::VideoCapture cap(0);
	cv::Mat frame;
	arMarkerRecognizer ar;

	if(!cap.isOpened()){
		ROS_INFO("Failed to open camera.");
		return -1;
	}

	ros::Rate rate(100);

	while(ar.nh_.ok()){
		if(cap.read(frame)){
			ar.recognizeArMarker(frame);
			ar.imagePublisher(frame);
		}

		ros::spinOnce();
		rate.sleep();
	}

	cap.release();
	cv::destroyAllWindows();

	return 0;
}
